/*******************************************************************************
 * Copyright (c) 2003, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     Matt Chapman - initial version
 *******************************************************************************/
package org.eclipse.contribution.visualiser.cvsprovider;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import org.eclipse.contribution.visualiser.core.Stripe;
import org.eclipse.contribution.visualiser.interfaces.IGroup;
import org.eclipse.contribution.visualiser.interfaces.IMember;
import org.eclipse.contribution.visualiser.simpleImpl.SimpleMarkupProvider;
import org.eclipse.contribution.visualiser.utils.MarkupUtils;

/**
 * Example implementation of a markup provider.  Loads its information about the markups from 
 * a file.  Manages the list of markups in a hashtable - the hashtable maps the member ID to
 * a list of 'Stripe instances' (markups).
 */
public class CVSMarkupProvider extends SimpleMarkupProvider {

//	private static Map colourMemory = new HashMap();
//	private static Map availableColours = new HashMap();
	
	// Indexed by FULL membername, each entry is a List of Stripe objects.
	public Hashtable markups = null;
	
	public SortedSet markupKinds;
	
	
	/**
	 * Initialise the markup provider.  This simple implementation does nothing here.
	 */
	public void initialise() { 
		markups = null;
		markupKinds = null;
	}


	/**
	 * Get a List of Stripes for the given member, which are its markups.
	 */
	public List getMemberMarkups(IMember member) {
		if(member != null && markups != null){	
			Object o = markups.get(member.getFullname());
			if(o != null){
				if(o instanceof List) {
					return (List) markups.get(member.getFullname());
				}
			}
		}
		return null;
	}


	/**
	 * Add a Stripe to the member with the given name.
	 * @param membername
	 * @param s
	 */
	public void addMarkup(String membername,Stripe s) {
		if (markups==null) markups = new Hashtable();
		List stripes = (List) markups.get(membername);
		if (stripes == null) {
			stripes = new ArrayList();
			stripes.add(s);
			markups.put(membername, stripes);
		} else {
			stripes.add(s);
		}
	}
	
	
	/**
	 * Process all the Stripes that have been added to deal with the overlapping cases
	 */
	public void processMarkups() {
	if(markups != null) {
		  Enumeration memkeys = markups.keys();
		  while (memkeys.hasMoreElements()) {
			 String memberID = (String)memkeys.nextElement();
			 List unprocessedListOfStripes = (List)markups.get(memberID);
			 MarkupUtils.processStripes(unprocessedListOfStripes);
		  }
	}
	}

	
	/**
	 * Get the markups for a group. Group markups are a stacked set of member markups.
	 */
	public List getGroupMarkups(IGroup group) {
		List stripes = new ArrayList();
		List kids = group.getMembers();
		int accumulatedOffset = 0;
		
		// Go through all the children of the group
		for (Iterator iter = kids.iterator(); iter.hasNext();) {
			IMember element = (IMember) iter.next();
			List l = (List) markups.get(element.getFullname());
			if (l != null) {
				for (Iterator iterator = l.iterator(); iterator.hasNext();) {
					Stripe elem = (Stripe) iterator.next();
					stripes.add(new Stripe(elem.getKinds(), elem.getOffset() + accumulatedOffset, elem.getDepth()));
				}
			}
			accumulatedOffset += element.getSize().intValue();
		}
		return stripes;
	}
	
	
	/**
	 * Get all the markup kinds.
	 * @return a Set of Strings
	 */
	public SortedSet getAllMarkupKinds() {
		// Created sorted list of markups
		if (markups==null) return null;
		if (markupKinds!=null) return markupKinds;
		markupKinds = new TreeSet();
		
		Enumeration stripeLists = markups.elements();
		while (stripeLists.hasMoreElements()) {
			List stripelist = (List)stripeLists.nextElement();
			for (Iterator iter = stripelist.iterator(); iter.hasNext();) {
				Stripe element = (Stripe) iter.next();
				markupKinds.addAll(element.getKinds());
			}
		}
		return markupKinds;
	}
	



	/**
	 * Process a mouse click on a stripe.  This implementation does nothing and returns
	 * true to allow the visualiser to perform the default operations.
	 * @see org.eclipse.contribution.visualiser.interfaces.IMarkupProvider#processMouseclick(java.lang.String, org.eclipse.contribution.visualiser.core.Stripe, int)
	 */
	public boolean processMouseclick(IMember member, Stripe stripe, int buttonClicked) {
		return true;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.contribution.visualiser.interfaces.IMarkupProvider#activate()
	 */
	public void activate() {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.contribution.visualiser.interfaces.IMarkupProvider#deactivate()
	 */
	public void deactivate() {
		// TODO Auto-generated method stub
		
	}



}
